/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.jni.*;

/**
 * This value class represents the CAN Channel Status. The CAN Channels Status
 * can be retrieved by calling {@link ICanChannel#GetStatus()}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public class CanChannelStatus
{
  /** current CAN line status */
  public CanLineStatus  m_oCanLineStatus  = new CanLineStatus();
  
  /**
   * TRUE if the channel is activated.
   * <p>
   * For {@link #toString()}: <code>"A"</code> means channel is activated
   */
  public boolean m_fActivated = false;

  /**
   * TRUE if receive FIFO overrun occurs
   * <p>
   * For {@link #toString()}: <code>"Q"</code> means receive FIFO overrun
   * occurred
   */
  public boolean m_fRxOverrun = false;
  
  /** receive FIFO load in percent (0..100) */
  public byte           m_bRxFifoLoad     = 0; 
  
  /** transmit FIFO load in percent (0..100)*/
  public byte           m_bTxFifoLoad     = 0; 
  
  /**
   * Creates a new CAN Channel Status object.
   * @throws Throwable
   */
  protected CanChannelStatus()
  {
  }
  
  /**
   * Creates a new CAN Channel Status object.
   * 
   * @param pCanChannelStatus
   *          CPointer to native memory which contains the CANCHANSTATUS
   *          structure.
   * @throws Throwable
   */
  protected CanChannelStatus(CPointer pCanChannelStatus) throws Throwable
  {
    fromCPointer(pCanChannelStatus);
  }
  
  /**
   * Updates a existing CAN Channel Status object.
   * 
   * @param pCanChannelStatus
   *          CPointer to native memory which contains the CANCHANSTATUS
   *          structure.
   * @throws Throwable
   */
  protected void fromCPointer(CPointer pCanChannelStatus) throws Throwable
  {
    int dwActivated = 0;
    int dwRxOverrun = 0;

    // Update CAN Line Status
    m_oCanLineStatus.fromCPointer(pCanChannelStatus);
    
    // Update CAN Channel Status
    dwActivated   = pCanChannelStatus.getInt(8);
    dwRxOverrun   = pCanChannelStatus.getInt(12);
    m_bRxFifoLoad = pCanChannelStatus.getByte(16);
    m_bTxFifoLoad = pCanChannelStatus.getByte(17);
    
    if(dwActivated != 0)
      m_fActivated = true;
    else
      m_fActivated = false;
    
    if(dwRxOverrun != 0)
      m_fRxOverrun = true;
    else
      m_fRxOverrun = false;
  }

  /**
   * Returns the CAN Channel Status as string.
   * @return The CAN Channel Status as string.
   */
  public String toString()
  {
    String strCanChannelStatus = "";
    
    strCanChannelStatus += "RxFifoLoad: " + m_bRxFifoLoad + "%, ";
    strCanChannelStatus += "TxFifoLoad: " + m_bTxFifoLoad + "%, ";
    strCanChannelStatus += "Flags: ";
    
    if(m_fActivated)
      strCanChannelStatus += "A";
    else
      strCanChannelStatus += " ";
    
    if(m_fRxOverrun)
      strCanChannelStatus += "O";
    else
      strCanChannelStatus += " ";

    return strCanChannelStatus;
  }

}
