/**
 * 
 */
package de.ixxat.vci3.bal.can;

import de.ixxat.vci3.IDisposable;
import de.ixxat.vci3.bal.BalSocketInfo;
import de.ixxat.vci3.bal.IBalResource;
import de.ixxat.vci3.bal.can.ICanChannel;

/**
 * This interface represents a CAN controller properties and capabilities
 * container. When no longer needed the CAN controller properties object has to
 * be disposed using the {@link IDisposable} interface. A CAN socket object can
 * be obtained via the method
 * {@link de.ixxat.vci3.bal.IBalObject#OpenSocket(int, int)}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public interface ICanSocket extends IBalResource
{
  
  /**
   * Gets information to the CAN Socket Object
   * 
   * @return {@link de.ixxat.vci3.bal.BalSocketInfo} - BalSocketInfo is used to
   *         signal the status of a CAN controller.
   * @throws Throwable
   */
  public BalSocketInfo GetSocketInfo() throws Throwable;
  
  /**
   * Gets information to the CAN Capabilities
   * 
   * @return {@link CanCapabilities} - CanCapabilities is used to get the
   *         capabilities of the CAN controller.
   * @throws Throwable
   */
  public CanCapabilities GetCapabilities() throws Throwable;
  
  /**
   * Gets the current status of the CAN Line.
   * 
   * @return {@link CanLineStatus} - CanLineStatus is used to signal the status of
   *         a CAN controller.
   * @throws Throwable
   */
  public CanLineStatus GetLineStatus() throws Throwable;

  /**
   * Creates a new CanChannel Object. After a successful creation you must call
   * {@link ICanChannel#Initialize(short, short)} to initialize the channel.
   * <p>
   * <strong>Remarks:</strong> <blockquote> The channel is deactivated after
   * this method returns an must be activated by an
   * {@link ICanChannel#Activate()} method call. </blockquote>
   * 
   * @param fExclusive
   *          If this parameter is set to true the method tries to initialize
   *          the channel for exclusive use. If set to false, the method
   *          initializes the channel for shared access.
   * @return {@link ICanChannel} - The new CanChannel Object.
   */
  public ICanChannel CreateChannel(boolean fExclusive) throws Throwable;

}

