/**
 * 
 */
package de.ixxat.vci3.bal.lin;

import de.ixxat.vci3.IDisposable;
import de.ixxat.vci3.bal.BalSocketInfo;
import de.ixxat.vci3.bal.IBalResource;

/**
 * This interface represents a LIN controller properties and capabilities
 * container. When no longer needed the LIN controller properties object has to
 * be disposed using the {@link IDisposable} interface. A LIN socket object can
 * be obtained via the method
 * {@link de.ixxat.vci3.bal.IBalObject#OpenSocket(int, int)}.
 * 
 * @author Michael Ummenhofer
 * 
 */
public interface ILinSocket extends IBalResource
{
  
  /**
   * Gets information to the CAN Socket Object
   * 
   * @return {@link de.ixxat.vci3.bal.BalSocketInfo} - BalSocketInfo is used to
   *         signal the status of a CAN controller.
   * @throws Throwable
   */
  public BalSocketInfo GetSocketInfo() throws Throwable;
  
  /**
   * Gets information to the LIN Capabilities
   * 
   * @return {@link LinCapabilities} - LinCapabilities is used to get the
   *         capabilities of the LIN controller.
   * @throws Throwable
   */
  public LinCapabilities GetCapabilities() throws Throwable;
  
  /**
   * Gets the current status of the LIN Line.
   * 
   * @return {@link LinLineStatus} - LinLineStatus is used to signal the status of
   *         a LIN controller.
   * @throws Throwable
   */
  public LinLineStatus GetLineStatus() throws Throwable;

  /**
   * Creates a new ILinMonitor Object. After a successful creation you must call
   * {@link ILinMonitor#Initialize(short)} to initialize the monitor.
   * <p>
   * <strong>Remarks:</strong> <blockquote> The monitor is deactivated after
   * this method returns an must be activated by an
   * {@link ILinMonitor#Activate()} method call. </blockquote>
   * 
   * @param fExclusive
   *          If this parameter is set to true the method tries to initialize
   *          the monitor for exclusive use. If set to false, the method
   *          initializes the monitor for shared access.
   * @return {@link ILinMonitor} - The new ILinMonitor Object.
   */
  public ILinMonitor CreateMonitor(boolean fExclusive) throws Throwable;

}

